// XcTest.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <windows.h>
#include <iostream>
#include <fstream>

#define TACCESS_API  __declspec(dllimport)
typedef bool (WINAPI *tcallback)(BYTE* pData);
typedef BYTE* (WINAPI *typeSendCommand)(BYTE* pData);
typedef bool (WINAPI  *typeFreeMemory)(BYTE* pData);
typedef bool (WINAPI *typeSetCallback)(tcallback pCallback);

typedef BYTE* (WINAPI *typeInitialize)(const BYTE* dir, int level);
typedef BYTE* (WINAPI *typeSetLogLevel)(int level);
typedef BYTE* (WINAPI *typeUninitialize)();

typeFreeMemory FreeMemory;

//////////////////////////////////////////////////////////////////////////
// ENCODING CONVERTERS
#include "stringbuffer.h"
void Utf8_2_ASCII(const char *str, std::string &output)
{
	if (!str) {
		output = "";
		return;
	}
	size_t len = strlen(str);
	if (len) {
		static WStringBuffer wbuff;
		static StringBuffer buff;
		wbuff.checkSize(len * 3);
		buff.checkSize(len * 3);
		MultiByteToWideChar(CP_UTF8, 0, str, -1, wbuff.getBuffer(), len * 3);
		WideCharToMultiByte(1251, 0, wbuff.getBuffer(), -1, buff.getBuffer(),
			buff.getSize(), 0, 0);
		output = buff.getBuffer();
	}
	else {
		output = "";
	}
}


void ASCII_2_Utf8(const char *str, std::string &output)
{
	size_t len = strlen(str);
	if (len) {
		static WStringBuffer wbuff;
		static StringBuffer buff;
		wbuff.checkSize(len * 3);
		buff.checkSize(len * 3);
		int char_written = MultiByteToWideChar(1251, 0, str, -1, wbuff.getBuffer(), len * 3);
		char_written = WideCharToMultiByte(CP_UTF8, 0, wbuff.getBuffer(), char_written, 0, 0, 0, 0);
		WideCharToMultiByte(CP_UTF8, 0, wbuff.getBuffer(), char_written, buff.getBuffer(),
			char_written + 1, 0, 0);
		output = buff.getBuffer();
	}
	else {
		output = "";
	}
}
//////////////////////////////////////////////////////////////////////////


void UnloadLibrary( HMODULE hm )
{
	try {
		FreeLibrary(hm);
	}
	catch (...) {
		std::cout<<"Fail in FreeLibrary"<<std::endl;
	}
}

bool CALLBACK acceptor(BYTE *pData)
{
	FreeMemory(pData);
	return true;
}

#ifndef XC64
	const char* lib_name = "txmlconnector.dll";
#else 
	const char* lib_name = "txmlconnector64.dll";
#endif

int main(int argc, char* argv[]) {
	const unsigned buffSize = 256;
	char error[buffSize] = {0};

	int mainResult = 0;
	setlocale(LC_CTYPE, "");

	HMODULE hm = LoadLibrary(lib_name);
	if (hm) {
		try
		{
			//    
			typeInitialize Initialize = 
				reinterpret_cast<typeInitialize>(GetProcAddress(hm, "Initialize"));
			if (!Initialize)	{
				sprintf_s(error, buffSize, "\"InitLog\" not found (0x%X)", GetLastError());
				throw std::runtime_error(error);
			}

			typeUninitialize UnInitialize = 
				reinterpret_cast<typeUninitialize>(GetProcAddress(hm, "UnInitialize"));
			if (!UnInitialize)	{
				sprintf_s(error, buffSize, "\"UnInitialize\" not found (0x%X)", GetLastError());
				throw std::runtime_error(error);
			}
	
			typeSetLogLevel SetLogLevel = 
				reinterpret_cast<typeSetLogLevel>(GetProcAddress(hm, "SetLogLevel"));
			if (!SetLogLevel)	{
				sprintf_s(error, buffSize, "\"SetLogLevel\" not found (0x%X)", GetLastError());
				throw std::runtime_error(error);
			}
	
			typeSetCallback SetCallback =
				reinterpret_cast<typeSetCallback>(GetProcAddress(hm, "SetCallback"));
			if (!SetCallback)	{
				sprintf_s(error, buffSize, "\"SetCallback\" not found (0x%X)", GetLastError());
				throw std::runtime_error(error);
			}
	
			typeSendCommand SendCommand =
				reinterpret_cast<typeSendCommand>(GetProcAddress(hm,"SendCommand"));
			if (!SendCommand)	{
				sprintf_s(error, buffSize, "\"SendCommand\" not found (0x%X)", GetLastError());
				throw std::runtime_error(error);
			}
	
			FreeMemory =
				reinterpret_cast<typeFreeMemory>(GetProcAddress(hm, "FreeMemory"));
			if (!FreeMemory)	{
				sprintf_s(error, buffSize, "\"FreeMemory\" not found (0x%X)", GetLastError());
				throw std::runtime_error(error);
			}

			//  
			int level = 3;
			std::cout<<"Initializing library: \".\\LOGS\"  level: "<< level <<std::endl;
			BYTE* res = Initialize(reinterpret_cast<const BYTE*>(".\\LOGS"), level);
	
			if (res) {
				std::string out;
				Utf8_2_ASCII(reinterpret_cast<const char*>(res), out);
				sprintf_s(error, buffSize, "Failed to initialize library: %s", out.c_str());
				FreeMemory(res);
				throw std::runtime_error(error);
			} else {
				std::cout<<"Library initialized successfully!"<<std::endl;
			}
	
			SetCallback(acceptor);
	
			std::cout<<"Sending \"connect\" command..."<<std::endl;
			BYTE* ss = SendCommand(reinterpret_cast<BYTE*>(
				"<command id='connect'>"
				"<login>KOKS</login><password>koks</password>"
				"<host>192.168.136.81</host><port>3901</port>"
				"<rqdelay>100</rqdelay>"
				"<logsdir>.\\LOGS\\</logsdir><loglevel>0</loglevel></command>"));
			std::string out;
			Utf8_2_ASCII(reinterpret_cast<const char*>(ss), out);
			std::cout<<out.c_str()<<std::endl;
			FreeMemory(ss);
	
			int c;
			printf( "Press ENTER to disconnect... " );
			fflush( stdout );
			do c = getchar(); while ((c != '\n') && (c != EOF));
	
			std::cout<<"Sending \"disconnect\" command..."<<std::endl;
			ss = SendCommand(reinterpret_cast<BYTE*>(
				"<command id='disconnect'/>"));
			std::cout<<reinterpret_cast<char*>(ss)<<std::endl;
			FreeMemory(ss);
	
			//   
			std::cout<<"Uninitializing library..." <<std::endl;
			res = UnInitialize();
			if (res) {
				std::string out;
				Utf8_2_ASCII(reinterpret_cast<const char*>(res), out);
				sprintf_s(error, buffSize, "Failed to uninitialize library: %s", out.c_str());
				FreeMemory(res);
				throw std::runtime_error(error);
			} else {
				std::cout<<"Library uninitialized successfully!"<<std::endl;
			}
		} catch (std::runtime_error& e) {
			std::cout<<"A fatal error occurred: "<<e.what()<<std::endl;
			UnloadLibrary(hm);
			mainResult = -1;
		}
	}
	int err = GetLastError();

	int c;
	printf( "Press ENTER to continue... " );
	fflush( stdout );
	do c = getchar(); while ((c != '\n') && (c != EOF));
	return mainResult;
}